#include "at_datafox_dfcom_spi_type2_DFCom.h"
#include "../AECommProject/AEComm.h"
#include <string.h>

char* getString(JNIEnv* env, jbyteArray arr) {
    jint len = env->GetArrayLength(arr);
    jbyte* arrbuffer = env->GetByteArrayElements(arr, 0);
    char* result = new char[len + 1];
    memcpy(result, arrbuffer, len);
    result[len] = 0;
    env->ReleaseByteArrayElements(arr, arrbuffer, JNI_ABORT);
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComOpenSerial
 * Signature: (ILjava/lang/String;II)Z
 */
JNIEXPORT jboolean JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComOpenSerial
(JNIEnv * env, jclass cls, jint channel, jbyteArray port, jint baud, jint timeout) {
    char* buffer = getString(env, port);
    jboolean result = DFCComOpenSerial(channel, reinterpret_cast<char*> (buffer), baud, timeout) ? JNI_TRUE : JNI_FALSE;
    delete[] buffer;
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComOpenSocket
 * Signature: (ILjava/lang/String;II)Z
 */
JNIEXPORT jboolean JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComOpenSocket
(JNIEnv * env, jclass cls, jint channel, jbyteArray socket, jint port, jint timeout) {
    char* buffer = getString(env, socket);
    jboolean result = DFCComOpenSocket(channel, buffer, port, timeout) ? JNI_TRUE : JNI_FALSE;
    delete[] buffer;
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComOpenIV
 * Signature: (IIILjava/lang/String;II)Z
 */
JNIEXPORT jboolean JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComOpenIV
(JNIEnv * env, jclass cls, jint channel, jint busid, jint type, jbyteArray comorthost, jint baudorport, jint timeout) {
    char* buffer = getString(env, comorthost);
    jboolean result = DFCComOpenIV(channel, busid, type, buffer, baudorport, timeout) ? JNI_TRUE : JNI_FALSE;
    delete[] buffer;
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComClose
 * Signature: (I)Z
 */
JNIEXPORT void JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComClose
(JNIEnv * env, jclass cls, jint channel) {
    DFCComClose(channel);
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComCheckDevice
 * Signature: (III)I
 */
JNIEXPORT jint JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcCheckDevice
(JNIEnv *, jclass, jint channel, jint bus, jint pollretry) {
    jint result;
    if (DFCCheckDevice(channel, bus, &result, pollretry)) {
        return 0;
    }
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComSetTime
 * Signature: (II[B)Z
 */
JNIEXPORT jboolean JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComSetTime
(JNIEnv * env, jclass, jint channel, jint bus, jbyteArray data) {
    jbyte* tdata = env->GetByteArrayElements(data, 0);
    jboolean result = DFCComSetTime(channel, bus, (unsigned char*) tdata) ? JNI_TRUE : JNI_FALSE;
    env->ReleaseByteArrayElements(data, tdata, JNI_ABORT);
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComGetTime
 * Signature: (II)[B
 */
JNIEXPORT jbyteArray JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComGetTime
(JNIEnv * env, jclass, jint channel, jint bus) {
    jbyteArray result = env->NewByteArray(7);
    jbyte* buffer = env->GetByteArrayElements(result, 0);
    if (DFCComGetTime(channel, bus, (unsigned char*) buffer)) {
        env->ReleaseByteArrayElements(result, buffer, JNI_COMMIT);
        return result;
    } else {
        env->ReleaseByteArrayElements(result, buffer, JNI_COMMIT);
        return 0;
    }
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComSendMessage
 * Signature: (IIIII[B)Z
 */
JNIEXPORT jboolean JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComSendMessage
(JNIEnv *env, jclass, jint channel, jint bus, jint delay, jint action, jint audio, jbyteArray msg) {
    char* string = getString(env, msg);
    jboolean result = DFCComSendMessage(channel, bus, delay, action, audio, string, strlen(string)) ? JNI_TRUE : JNI_FALSE;
    delete[] string;
    return result;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcComSendInfoText
 * Signature: (II[B)Z
 */
JNIEXPORT jboolean JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcComSendInfoText
(JNIEnv * env, jclass, jint channel, jint bus, jbyteArray msg) {
    char* string = getString(env, msg);
    jboolean result = DFCComSendInfotext(channel, bus, string, strlen(string));
    delete[] string;
    return result;
}

jobject createSerialNumberResult(JNIEnv* env, jlong error, jlong result) {
    jclass clazz = env->FindClass("at/datafox/dfcom/spi/type2/DFCom$SerialNumberResult");
    if (clazz == 0) {
        return 0;
    }
    jmethodID cid = env->GetMethodID(clazz, "<init>", "(JJ)V");
    if (cid == 0) {
        return 0;
    }
    jobject obj = env->NewObject(clazz, cid, error, result);
    env->DeleteLocalRef(clazz);
    return obj;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcGetSeriennummer
 * Signature: (II)Lat/datafox/dfcom/spi/type2/DFCom$SerialNumberResult;
 */
JNIEXPORT jobject JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcGetSeriennummer
(JNIEnv * env, jclass, jint channel, jint bus) {
    int error = 0, serial = 0;
    jobject result;
    if (DFCGetSeriennummer(channel, bus, &error, &serial)) {
        return createSerialNumberResult(env, 0, serial);
    }
    return createSerialNumberResult(env, error, serial);
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcSetLog
 * Signature: (IZ)V
 */
JNIEXPORT void JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcSetLog
(JNIEnv *, jclass, jint channel, jboolean on) {
    if (on == JNI_TRUE) {
        DFCSetLogOn(channel);
    } else {
        DFCSetLogOff(channel);
    }
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcSetLogFilename
 * Signature: (I[B)V
 */
JNIEXPORT void JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcSetLogFilename
(JNIEnv * env, jclass, jint channel, jbyteArray barray) {
    char* logFile = getString(env, barray);
    DFCSetLogFileName(channel, (unsigned char*) logFile);
    delete[] logFile;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcSetCallBack
 * Signature: (ILat/datafox/dfcom/spi/type2/DFCom$CallBack;)V
 */
JNIEXPORT void JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcSetCallBack
(JNIEnv *, jclass, jint, jobject) {
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcSetGlobVar
 * Signature: (II[BI[B)I
 */
JNIEXPORT jint JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcSetGlobVar
(JNIEnv *, jclass, jint, jint, jbyteArray, jint, jbyteArray) {
    return 0;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcGetGlobVar
 * Signature: (II[BI)Lat/datafox/dfcom/spi/type2/DFCom$GetGlobalVarResult;
 */
JNIEXPORT jobject JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcGetGlobVar
(JNIEnv *, jclass, jint, jint, jbyteArray, jint) {
    return 0;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcCloseRelay
 * Signature: (IIBI)I
 */
JNIEXPORT jint JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcCloseRelay
(JNIEnv * env, jclass, jint channel, jint bus, jbyte relay, jint time) {
    jint result;
    if (DFCCloseRelay(channel, bus, relay, time, &result)) {
        return 0;
    }
    return result;
}

jobject createGetRelayStateResult(JNIEnv* env, jlong error, jint time, jboolean state) {
    jclass clazz = env->FindClass("at/datafox/dfcom/spi/type2/DFCom$GetRelayStateResult");
    if (clazz == 0) {
        return 0;
    }
    jmethodID cid = env->GetMethodID(clazz, "<init>", "(JIZ)V");
    if (cid == 0) {
        return 0;
    }
    jobject obj = env->NewObject(clazz, cid, error, time, state);
    env->DeleteLocalRef(clazz);
    return obj;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcGetRelayState
 * Signature: (IIB)Lat/datafox/dfcom/spi/type2/DFCom$GetRelayStateResult;
 */
JNIEXPORT jobject JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcGetRelayState
(JNIEnv * env, jclass, jint channel, jint bus, jbyte relay) 
{
    int error = 0;
    int time = 0;
    int state = 0;
    DFCGetRelayState(channel,bus,relay,&state,&time,&error);
    return createGetRelayStateResult(env,error,time,state?JNI_TRUE:JNI_FALSE);
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcOpenRelay
 * Signature: (IIB)I
 */
JNIEXPORT jint JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcOpenRelay
(JNIEnv *, jclass, jint, jint, jbyte)
{
    return 0;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcGetDevicePollRetry
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcGetDevicePollRetry
(JNIEnv *, jclass, jint)
{
    return 0;
}

/*
 * Class:     at_datafox_dfcom_spi_type2_DFCom
 * Method:    dfcGetVersionFirmware
 * Signature: (II)Lat/datafox/dfcom/spi/type2/DFCom$GetVersionFirmwareResult;
 */
JNIEXPORT jobject JNICALL Java_at_datafox_dfcom_spi_type2_DFCom_dfcGetVersionFirmware
(JNIEnv *, jclass, jint, jint)
{
    return 0;
}
