/**
 * @file  DFLog.h
 * @brief Schnittstelle fuer die Klasse CDFLog.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFLOG_H_
#define _DFLOG_H_

#include <time.h>
#include <string>

// Initialisierungsdatei verwalten
#include "../File/fileini.h"
// Hilfsklasse zur Verwaltung alter Logdateien
#include "DFLogCheck.h"

#ifdef _MSC_VER
#include <windows.h>
#else
#include </usr/include/pthread.h>
#endif

/* Die Logklasse ist fuer ein standardmaessiges logen von Funktionsaufrufen,
 * so dass Fehler in der Verarbeitung auch noch nach einigen Tagen 
 * nachfolzogen werden koennten.
 * Sie ist ueber eine Initialisierungsdatei mit grundinformationen versorgbar.
 *
 */
#define SUB_FOLDER           "DFComLog"

#define INI_FILENAME         "DFCom.ini"
// YYYY_MM_DD_DFComDLL_channelID.log
#define LOG_FILENAME         "%s%s20%02i_%02i_%02i_DFComDLL_%i.log"
#define LOG_SEARCH_FILENAME  "%s%s20??_??_??_DFComDLL_*.log"
#define LOG_DELETE_FILENAME  "%s%s%s"

#ifdef _MSC_VER
#define PATH_BASE            ".\\"
#else
#define PATH_BASE            "./"
#endif

#ifdef _MSC_VER
#define fullpath             _fullpath
#define mkdir                _mkdir
#define stat                 _stat
#define finddata_t           _finddata_t
#define findfirst            _findfirst
#define findnext             _findnext
#define findclose            _findclose
#endif

class CDFLog
{
	bool m_bEnable;                  // Log aktiv?
	char m_szFileName[260];          // Dateiname der Logdatei
	bool m_bEraseFiles;              // Alte Dateien loeschen
	bool m_bElapse;                  // Ausfuehrungszeit angeben
	int m_bWriteList;                // Listendaten mitloggen
public:
    int _dockingPreferType;          // Bevorzugte Version der Dockingstation
    int _dockingAutoDetect;          // Automatisch die Version der Dockingstation ermitteln, es wird mit _preverVersion begonnen.
	int m_traceMode;                 // Tracing aktivieren
	char m_szFileNameTrace[260];     // Dateiname der Tracedatei

public:
	CDFLog(short channelID);
	virtual ~CDFLog();

	// Ist Log aktiv?
	inline bool logEnabled() { return m_bEnable; }

	// Erzeugen einer Pfadangabe
	void MakeFilePath(char *path_buffer, char *szFileName, char *szFileExt);

	// Einlesen der Initialisierungsdatei
	void logIniRead();
	// Pruefen auf alte Dateien.
	void logCheck();

	// Schreiben einer Logzeile, wenn Log aktiv.
	void logBegin(const char *Format, ...); // Mit Uhrzeit
	void logAppend(const char *szFormat, ...);
	void logEnd(const char *szFormat, ...); // Mit Zeilenumbruch
	char *logGetBasePath(char *basePath)
	{
		sprintf(basePath, "%s%s", m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder);
		return basePath;
	}
	int logIsWriteList() { return m_bEnable ? m_bWriteList : 0; }
	// Setzen des Namen fuer Tracefile
	virtual void setTraceFileName( const std::string &filePath )
    {
        (void)filePath;
    }

protected:
	// Zugewiesene Kanalnummer
	short m_channelID;
	// Zu verwendende Initialisierungsdatei
	FileIni m_logIni;
	// Start des Logvorganges
	clock_t m_clockBegin;
	// Alte Logdateien verwalten
	CDFLogCheck m_logCheck;

#ifdef _MSC_VER
    CRITICAL_SECTION m_mutexFile;
#else
    pthread_mutex_t m_mutexFile;
#endif
};

#endif

